
#include <avr/io.h>
#include <inttypes.h>
#include <pgmspace.h>
#include <avr/sleep.h>

#include <spi.h>
#include <can.h>
#include <mcp2515.h>
#include <display.h>

#include <scheduler.h>
#include <buskoppler.h>


/************************************************************************/
/* Set Reset:
/* Software-Reset des MCP2515
/************************************************************************/
const uint8_t mcp2515_bef_reset[]    PROGMEM = {1, MCP2515_RESET};

/************************************************************************/
/* Set CAN-Ctrl:
/* 0x44 Loopback-Mode, Clockout-Pin enabled, Fclkout = System-Clk / 2
/* 0x04 Normal-Mode,   Clockout-Pin enabled, Fclkout = System-Clk / 2
/*
/************************************************************************/
const uint8_t mcp2515_bef_configmode[] PROGMEM = {3, MCP2515_WRITE, MCP2515_CANCTRL, 0x84};
//const uint8_t mcp2515_bef_readclk[]    PROGMEM = {3, MCP2515_READ,  MCP2515_CANCTRL, 0x00};

/************************************************************************/
/* Set Config-Register CNF3, CNF2, CNF1:
/* CNF1 =
/*
/************************************************************************/

// beim 1. MCP2515 ist SOF low: (Clock-Output eingeschaltet)
// const uint8_t mcp2515_bef0_conf666k[]  PROGMEM = {5, MCP2515_WRITE, MCP2515_CNF3, 0x01, 0x80, 0x00};
const uint8_t mcp2515_bef0_conf500k[]  PROGMEM = {5, MCP2515_WRITE, MCP2515_CNF3, 0x01, 0x92, 0x00};
// const uint8_t mcp2515_bef0_conf400k[]  PROGMEM = {5, MCP2515_WRITE, MCP2515_CNF3, 0x02, 0x92, 0x40};
// const uint8_t mcp2515_bef0_conf333k[]  PROGMEM = {5, MCP2515_WRITE, MCP2515_CNF3, 0x03, 0x92, 0x80};
const uint8_t mcp2515_bef0_conf250k[]  PROGMEM = {5, MCP2515_WRITE, MCP2515_CNF3, 0x04, 0x9B, 0xC0};
// const uint8_t mcp2515_bef0_conf125k[]  PROGMEM = {5, MCP2515_WRITE, MCP2515_CNF3, 0x04, 0x9B, 0xC1};
// const uint8_t mcp2515_bef0_conf100k[]  PROGMEM = {5, MCP2515_WRITE, MCP2515_CNF3, 0x04, 0xA4, 0xC1};
// const uint8_t mcp2515_bef0_conf50k[]   PROGMEM = {5, MCP2515_WRITE, MCP2515_CNF3, 0x04, 0xA4, 0xC3};
const uint8_t mcp2515_bef0_conf25k[]   PROGMEM = {5, MCP2515_WRITE, MCP2515_CNF3, 0x04, 0xA4, 0xC7};
// const uint8_t mcp2515_bef0_conf10k[]   PROGMEM = {5, MCP2515_WRITE, MCP2515_CNF3, 0x04, 0xA4, 0xD3};

// beim 2. MCP2515 ist SOF high: (um kein 2. Clock-Signal zu generieren)
// const uint8_t mcp2515_bef1_conf666k[]  PROGMEM = {5, MCP2515_WRITE, MCP2515_CNF3, 0x81, 0x80, 0x00};
const uint8_t mcp2515_bef1_conf500k[]  PROGMEM = {5, MCP2515_WRITE, MCP2515_CNF3, 0x81, 0x92, 0x00};
// const uint8_t mcp2515_bef1_conf400k[]  PROGMEM = {5, MCP2515_WRITE, MCP2515_CNF3, 0x82, 0x92, 0x40};
// const uint8_t mcp2515_bef1_conf333k[]  PROGMEM = {5, MCP2515_WRITE, MCP2515_CNF3, 0x83, 0x92, 0x80};
const uint8_t mcp2515_bef1_conf250k[]  PROGMEM = {5, MCP2515_WRITE, MCP2515_CNF3, 0x84, 0x9B, 0xC0};
// const uint8_t mcp2515_bef1_conf125k[]  PROGMEM = {5, MCP2515_WRITE, MCP2515_CNF3, 0x84, 0x9B, 0xC1};
// const uint8_t mcp2515_bef1_conf100k[]  PROGMEM = {5, MCP2515_WRITE, MCP2515_CNF3, 0x84, 0xA4, 0xC1};
// const uint8_t mcp2515_bef1_conf50k[]   PROGMEM = {5, MCP2515_WRITE, MCP2515_CNF3, 0x84, 0xA4, 0xC3};
const uint8_t mcp2515_bef1_conf25k[]   PROGMEM = {5, MCP2515_WRITE, MCP2515_CNF3, 0x84, 0xA4, 0xC7};
// const uint8_t mcp2515_bef1_conf10k[]   PROGMEM = {5, MCP2515_WRITE, MCP2515_CNF3, 0x84, 0xA4, 0xD3};

const uint8_t mcp2515_bef_conf_tpins[] PROGMEM = {3, MCP2515_WRITE, MCP2515_TXRTSCTRL, 0x07};
const uint8_t mcp2515_bef_conf_rbuf0[] PROGMEM = {3, MCP2515_WRITE, MCP2515_RXB0CTRL,  0x60};
const uint8_t mcp2515_bef_conf_rbuf1[] PROGMEM = {3, MCP2515_WRITE, MCP2515_RXB1CTRL,  0x60};

const uint8_t mcp2515_bef_conf_mask0[] PROGMEM = {6, MCP2515_WRITE, MCP2515_RXM0SIDH, 0,0,0,0};
const uint8_t mcp2515_bef_conf_mask1[] PROGMEM = {6, MCP2515_WRITE, MCP2515_RXM1SIDH, 0,0,0,0};

const uint8_t mcp2515_bef_conf_int[]   PROGMEM = {4, MCP2515_WRITE, MCP2515_CANINTE, 0x1F,0};
const uint8_t mcp2515_bef_conf_rpins[] PROGMEM = {3, MCP2515_WRITE, MCP2515_BFPCTRL, 0x0C};
const uint8_t mcp2515_bef_normalmode[] PROGMEM = {3, MCP2515_WRITE, MCP2515_CANCTRL, 0x04};  // 0x04 normal! 0x0C One-Shot-Mode

const uint8_t mcp2515_bef_pwdnmode[]   PROGMEM = {3, MCP2515_WRITE, MCP2515_CANCTRL, 0x67};  // 0x04 normal! 0x0C One-Shot-Mode

const uint8_t mcp2515_bef_clr_TX0IF[]  PROGMEM = {4, MCP2515_BITMOD,MCP2515_CANINTF, 0x04, 0};  // 0x04 normal!

const uint8_t mcp2515_bef_wfilter0[]   PROGMEM = {8, MCP2515_WRITE, MCP2515_RXF0SIDH, 8, 9, 10, 11};
const uint8_t mcp2515_bef_rfilter0[]   PROGMEM = {6, MCP2515_READ,  MCP2515_RXF0SIDH, 255, 255, 255, 255};

const uint8_t mcp2515_bef_set_int[]    PROGMEM = {4, MCP2515_WRITE, MCP2515_CANINTE, 0x01, 0x04};


/************************************************************************/
/*
/* function delay_us
/*
/* Fr kurze Verzgerungen im us/ms-Bereich.
/* Ausgelegt fr einen 8Mhz-Quarz.
/*
/************************************************************************/
void delay_us(uint16_t us){
  do{
    asm("nop");  	// 4* je 1 Cycle
    asm("nop");
    asm("nop");
    asm("nop");
  } while (--us);   	// 4 Cycles
}


void delay_ms(uint16_t us){
  uint16_t i;
  for(i=0;i<1000;i++){
    delay_us(1000);
  }
}

/*
void taster_init(void){
  tast_a_ddr  &= ~tast_a_mask;
  tast_a_port |= tast_a_mask; 	// Pullups einschalten
  tast_c_ddr  &= ~tast_c_mask;
  tast_c_port |= tast_c_mask;
}

uint8_t get_taster(void){
  return( ~(((tast_a_pin & tast_a_mask) >> 2) | (tast_c_pin & tast_c_mask)) );
}

uint8_t testbuf[132];
uint8_t *testptr = testbuf;
*/


int main(void){
//  volatile uint8_t c;
//  volatile uint16_t i;
//  uint8_t j;
//  uint8_t spi_maindata;
  uint8_t mcp_buf[14];
//  uint8_t status_buf0[8];
//  uint8_t status_buf1[8];



  reset_ddr      |=  _BV(reset_bit);			// Hardware-Reset = high
  reset_port     &=  ~_BV(reset_bit);

  led_ddr        |=  _BV(led_bit);
  led_port       |=  _BV(led_bit);

  PORTA          |= 0xFC;		// Pullups an PORTA einschalten
  PORTC          |= 0xC0;		// Pullups an PORTC einschalten

//  TCCR0         =   (_BV(WGM00) | _BV(WGM01) | _BV(COM01) | _BV(COM00) | _BV(CS01));
//  OCR0	        =   0xC0;
//  pwm_ddr       |=  _BV(pwm_bit);  			// PWM-Pin als Ausgang

  drv_pwdn1_ddr  |=  _BV(drv_pwdn1_bit);
  drv_pwdn1_port &= ~_BV(drv_pwdn1_bit);
  drv_pwdn2_ddr  |=  _BV(drv_pwdn2_bit);
  drv_pwdn2_port &= ~_BV(drv_pwdn2_bit);

/*
  taster_init();
*/

  spi_master_init();


  spi_const_mcp2515_0(mcp_buf, mcp2515_bef_reset);	// Software-Reset an MCP2515
  spi_const_mcp2515_1(mcp_buf, mcp2515_bef_reset);	// Software-Reset an MCP2515
  delay_us(15000);					// warten, bis MCP2515 Reset verarbeitet hat
  reset_port |= _BV(reset_bit);				// externen Reset (PortD.6) beenden

  spi_const_mcp2515_0(mcp_buf, mcp2515_bef_configmode);	// Clockausgang 8Mhz fr AVR setzen
  spi_const_mcp2515_0(mcp_buf, mcp2515_bef0_conf25k);
  spi_const_mcp2515_0(mcp_buf, mcp2515_bef_conf_tpins);	// TXxRTS-Pins als Input (interner Pullup 100k)

  spi_const_mcp2515_0(mcp_buf, mcp2515_bef_conf_rbuf0);	// switch Mask 0 off !!!
  spi_const_mcp2515_0(mcp_buf, mcp2515_bef_conf_rbuf1);	// switch Mask 1 off !!!
  spi_const_mcp2515_0(mcp_buf, mcp2515_bef_conf_mask0);	// Mask 0 dont care
  spi_const_mcp2515_0(mcp_buf, mcp2515_bef_conf_mask1);	// Mask 1 dont care
  spi_const_mcp2515_0(mcp_buf, mcp2515_bef_conf_int);	// Interrupt-Mask off
  spi_const_mcp2515_0(mcp_buf, mcp2515_bef_conf_rpins);	// RXxBF-Pins als Output

  spi_const_mcp2515_1(mcp_buf, mcp2515_bef_configmode);	// Clockausgang 8Mhz fr AVR setzen
  spi_const_mcp2515_1(mcp_buf, mcp2515_bef1_conf25k);
  spi_const_mcp2515_1(mcp_buf, mcp2515_bef_conf_tpins);	// TXxRTS-Pins als Input (interner Pullup 100k)

  spi_const_mcp2515_1(mcp_buf, mcp2515_bef_conf_rbuf0);	// switch Mask 0 off !!!
  spi_const_mcp2515_1(mcp_buf, mcp2515_bef_conf_rbuf1);	// switch Mask 1 off !!!
  spi_const_mcp2515_1(mcp_buf, mcp2515_bef_conf_mask0);	// Mask 0 dont care
  spi_const_mcp2515_1(mcp_buf, mcp2515_bef_conf_mask1);	// Mask 1 dont care
  spi_const_mcp2515_1(mcp_buf, mcp2515_bef_conf_int);	// Interrupt-Mask off
  spi_const_mcp2515_1(mcp_buf, mcp2515_bef_conf_rpins);	// RXxBF-Pins als Output

  spi_const_mcp2515_0(mcp_buf, mcp2515_bef_normalmode);	// Konfiguration beenden
  spi_const_mcp2515_1(mcp_buf, mcp2515_bef_normalmode);	// Konfiguration beenden

  spi_const_mcp2515_0(mcp_buf, mcp2515_bef_set_int);


  GICR = 0x40;
  can_init_msgs();


//  display_init();


  scheduler();
}


/*
  while(1){
    if(get_taster() != 0){
      spi_const_mcp2515_1(mcp_buf, mcp2515_bef_pwdnmode);	// RXxBF-Pins als Output
      spi_const_mcp2515_0(mcp_buf, mcp2515_bef_pwdnmode);	// RXxBF-Pins als Output
      drv_pwdn_port |= _BV(drv_pwdn_bit);

      set_sleep_mode(SLEEP_MODE_IDLE);
      sleep_mode();

      while ((c = get_taster()) != 0);
    }

    drv_pwdn_port &= ~_BV(drv_pwdn_bit);
    spi_const_mcp2515_0(mcp_buf, mcp2515_bef_normalmode);	// RXxBF-Pins als Output
    delay_us(200);
    spi_const_mcp2515_1(mcp_buf, mcp2515_bef_normalmode);	// RXxBF-Pins als Output
    delay_us(200);

    mcp_buf[1] =  0x00;
    mcp_buf[2] |= 0x08;
    mcp_buf[3] = 0;
    mcp_buf[4] = 0;
    mcp_buf[5] = 8;
    mcp_buf[6] = j++;

    for (i=1; i<8; i++){
      if (i & 0x01){
        mcp_buf[6+i] = 0;
      }
      else{
        mcp_buf[6+i] = 0x0ff;
      }
    }
    mcp0_txd_buf0(mcp_buf);

    status_buf0[0] = MCP2515_READ;
    status_buf0[1] = MCP2515_CANINTE;
    spi_mcp2515_0(status_buf0, 7);

    delay_us(20000);
    spi_const_mcp2515_0(mcp_buf, mcp2515_bef_clr_TX0IF);

    status_buf0[0] = MCP2515_READ;
    status_buf0[1] = MCP2515_CANINTE;
    spi_mcp2515_0(status_buf0, 7);

    status_buf1[0] = MCP2515_READ;
    status_buf1[1] = MCP2515_CANINTE;
    spi_mcp2515_1(status_buf1, 7);

//    memcpy_P(mcp_buf, mcp2515_bef_wfilter0, 6);
//    spi_mcp2515_0(mcp_buf,6);
//    memcpy_P(mcp_buf, mcp2515_bef_rfilter0, 6);
//    spi_mcp2515_0(mcp_buf,6);

  }
}
*/

