
#include <avr/interrupt.h>
#include <avr/signal.h>

#include <avr/io.h>
#include <inttypes.h>
#include <pgmspace.h>

#include <spi.h>
#include <mcp2515.h>
#include <can.h>
#include <buskoppler.h>


/************************************************************************/
/*
/*
/*
/************************************************************************/

#define can_txd0_buf_size	16

can_msg_typ can_txd0_msg_buf[can_txd0_buf_size];
can_msg_typ *can_txd0_in_ptr;
can_msg_typ *can_txd0_out_ptr;
volatile uint8_t can_txd0_cnt;


#define can_rxd0_buf_size	16

can_msg_typ can_rxd0_msg_buf[can_rxd0_buf_size];
can_msg_typ *can_rxd0_in_ptr;
can_msg_typ *can_rxd0_out_ptr;
volatile uint8_t can_rxd0_cnt;

#define can_rxd1_buf_size	8

can_msg_typ can_rxd1_msg_buf[can_rxd1_buf_size];
can_msg_typ *can_rxd1_in_ptr;
can_msg_typ *can_rxd1_out_ptr;
volatile uint8_t can_rxd1_cnt;

volatile uint8_t can_interrupt_enable_merker;


/************************************************************************/
/*
/*
/*
/************************************************************************/

uint8_t can_get_status(void){
  volatile uint8_t mask_status, flag_status;;
  //*** Status holen / Quelle des IR erkunden: ******************
  cli();
  spi_port &= ~_BV(spi_cs0_bit);
  SPDR = MCP2515_READ;
  while(!(SPSR & _BV(SPIF)));
  SPDR = MCP2515_CANINTE;
  while(!(SPSR & _BV(SPIF)));
  SPDR = 0;
  while(!(SPSR & _BV(SPIF)));
  mask_status = SPDR;
  SPDR = 0;
  while(!(SPSR & _BV(SPIF)));
  flag_status = SPDR;
  spi_port |= _BV(spi_cs0_bit);
  sei();
  return(flag_status);
}


void can_init_msgs(void){
  can_txd0_in_ptr  = &can_txd0_msg_buf[0];
  can_txd0_out_ptr = &can_txd0_msg_buf[0];
  can_txd0_cnt     = 0;

  can_rxd0_in_ptr  = &can_rxd0_msg_buf[0];
  can_rxd0_out_ptr = &can_rxd0_msg_buf[0];
  can_rxd0_cnt     = 0;

  can_rxd1_in_ptr  = &can_rxd1_msg_buf[0];
  can_rxd1_out_ptr = &can_rxd1_msg_buf[0];
  can_rxd1_cnt     = 0;
  can_interrupt_enable_merker = 1;
}


/************************************************************************/
/*
/* can_msg_typ *can_assign_txd0_msg(void)
/*
/* gibt einen Ptr auf eine freie can_txd0_msg zurck.
/* Falls der txd0-Puffer voll ist, wird gewartet.
/*
/* Der Versand der msg erfolgt separat mit can_put_txd0_msg(void)!
/*
/************************************************************************/

can_msg_typ *can_assign_txd0_msg(void){
  can_msg_typ *msg_ptr = can_txd0_in_ptr;
  while (can_txd0_cnt >= can_txd0_buf_size);
  if (++can_txd0_in_ptr > &can_txd0_msg_buf[can_txd0_buf_size-1]){
    can_txd0_in_ptr = &can_txd0_msg_buf[0];
  }
  return(msg_ptr);
}


/************************************************************************/
/*
/* void can_put_txd0_msg(void)
/*
/* Versendet eine can_txd0_msg.
/* Der Ptr auf die msg muss zwingend vorher mit can_get_txd0_msg
/*
/************************************************************************/

void can_put_txd0_msg(void){
  cli();
  if(can_txd0_cnt++ == 0){
    spi_port &= ~_BV(spi_cs0_bit);
    SPDR = MCP2515_BITMOD;		// TXD-IR enablen:
    while(!(SPSR & _BV(SPIF)));
    SPDR = MCP2515_CANINTE;		// Adresse
    while(!(SPSR & _BV(SPIF)));
    SPDR = MCP2515_TX0IE;		// Maske
    while(!(SPSR & _BV(SPIF)));
    SPDR = 0xff;  // MCP2515_TX0IE;		// Bit setzen
    while(!(SPSR & _BV(SPIF)));
    spi_port |= _BV(spi_cs0_bit);
    can_interrupt_enable_merker |= MCP2515_TX0IE;
  }
  sei();
}


/************************************************************************/
/*
/* can_msg_typ *can_get_rxd0_msg(void)
/*
/* Holt eine msg aus dem Eingangsbuffer.
/* Ob eine msg im Eingangsbuffer vorhanden ist, muss extern sichergestellt
/* werden!
/* Nach dem Verarbeiten der msg muss die msg mit can_free_rxd0_msg
/* wieder freigegeben werden.
/*
/************************************************************************/

can_msg_typ *can_get_rxd0_msg(void){
  can_msg_typ *msg_ptr = can_rxd0_out_ptr;
//  while (!can_rxd0_cnt);
  if (++can_rxd0_out_ptr > &can_rxd0_msg_buf[can_rxd0_buf_size-1]){
    can_rxd0_out_ptr = &can_rxd0_msg_buf[0];
  }
  return(msg_ptr);
}


void can_free_rxd0_msg(void){
  can_rxd0_cnt--;
}


/************************************************************************/
/*
/* uint8_t can_assign_rxd0_msg(can_msg_typ my_msg_ptr)
/*
/* gibt einen Ptr auf eine freie can_rxd0_msg zurck.
/* Falls der rxd0-Puffer voll ist, wird die Aktion abgebrochen und als
/* Ergebnis NULL (Buffer voll) geliefert.
/* Wenn eine rxd0-msg frei ist, wird als Ergebnis 1 (OK) geliefert.
/*
/* Fr den Versand einer msg fr den Buskoppler-internen Gebrauch in den
/* Eingangspuffer (!)
/*
/* Ein separates can_put_rxd0_msg() ist nicht notwendig.
/* Der Eingangspuffer darf von main() nicht getestet werden, bevor die
/* zurckgelieferte msg gefllt wurde!
/*
/************************************************************************/

uint8_t can_assign_rxd0_msg(can_msg_typ **my_msg_ptr){
  cli();
  if (can_rxd0_cnt < can_rxd0_buf_size){
    *my_msg_ptr = can_rxd0_in_ptr;
    if (++can_rxd0_in_ptr > &can_rxd0_msg_buf[can_rxd0_buf_size-1]){
      can_rxd0_in_ptr = &can_rxd0_msg_buf[0];
    }
    can_rxd0_cnt++;
    sei();
    return(1);
  }
  else{
    sei();
    return(0);
  }
}


/************************************************************************/
/*
/*
/*
/************************************************************************/

SIGNAL(SIG_INTERRUPT0){
  uint8_t status;
  uint8_t i, anz;
  volatile uint8_t *buf_ptr;

  //*** Status holen / Quelle des IR erkunden: ***************************

  spi_port &= ~_BV(spi_cs0_bit);
  SPDR = MCP2515_READ_STATUS;
  while(!(SPSR & _BV(SPIF)));
  SPDR = 0;
  while(!(SPSR & _BV(SPIF)));
  status = SPDR;
  spi_port |= _BV(spi_cs0_bit);

  //*** Receive0-IR: *****************************************************

  if (status & 0x01){
    spi_port &= ~_BV(spi_cs0_bit);
    SPDR = MCP2515_READ_RX0;
    buf_ptr = &((*can_rxd0_out_ptr)[0]);
    while(!(SPSR & _BV(SPIF)));
    // CAN-ID und Size holen:
    for(i=0; i<5; i++){
      SPDR = 0;
      while(!(SPSR & _BV(SPIF)));
      *buf_ptr++ = SPDR;
    }
    anz = SPDR & 0xF;
    if (anz > 8) anz = 8;
    for(i=0; i<anz; i++){
      SPDR = 0;
      while(!(SPSR & _BV(SPIF)));
      *buf_ptr++ = SPDR;
    }
    spi_port |= _BV(spi_cs0_bit);
    if (++can_rxd0_in_ptr > &can_rxd0_msg_buf[can_rxd0_buf_size-1]){
      can_rxd0_in_ptr = &can_rxd0_msg_buf[0];
    }
    can_rxd0_cnt++;
  }

 //*** Receive1-IR: ******************************************************

  if (status & 0x02){
    spi_port &= ~_BV(spi_cs0_bit);
    SPDR = MCP2515_READ_RX1;
    buf_ptr = &((*can_rxd1_in_ptr)[0]);
    while(!(SPSR & _BV(SPIF)));
    // CAN-ID und Size holen:
    for(i=0; i<5; i++){
      SPDR = 0;
      while(!(SPSR & _BV(SPIF)));
      *buf_ptr++ = SPDR;
    }
    anz = SPDR & 0xF;
    if (anz > 8) anz = 8;
    for(i=0; i<anz; i++){
      SPDR = 0;
      while(!(SPSR & _BV(SPIF)));
      *buf_ptr++ = SPDR;
    }
    spi_port |= _BV(spi_cs0_bit);
    if (++can_rxd1_in_ptr > &can_rxd1_msg_buf[can_rxd1_buf_size-1]){
      can_rxd1_in_ptr = &can_rxd1_msg_buf[0];
    }
    can_rxd1_cnt++;
  }

  //*** Transmit0-Empty-IR: **********************************************

  if ((status & 0x08) && (can_interrupt_enable_merker & MCP2515_TX0IE)){

    spi_port &= ~_BV(spi_cs0_bit);
    SPDR = MCP2515_BITMOD;
    while(!(SPSR & _BV(SPIF)));
    SPDR = MCP2515_CANINTF;
    while(!(SPSR & _BV(SPIF)));
    SPDR = MCP2515_TX0IF;
    while(!(SPSR & _BV(SPIF)));
    SPDR = 0;
    while(!(SPSR & _BV(SPIF)));
    spi_port |= _BV(spi_cs0_bit);

    spi_port &= ~_BV(spi_cs0_bit);
    SPDR = MCP2515_LOAD_TX0;
    buf_ptr = &((*can_txd0_out_ptr)[0]);
    anz = *(buf_ptr+4) + 5;
    while(!(SPSR & _BV(SPIF)));
    for(i=0; i<anz; i++){
      SPDR = *buf_ptr++;
      while(!(SPSR & _BV(SPIF)));
    }
    spi_port |= _BV(spi_cs0_bit);

    spi_port &= ~_BV(spi_cs0_bit);
    SPDR = MCP2515_RTS_TX0;
    if (++can_txd0_out_ptr > &can_txd0_msg_buf[can_txd0_buf_size-1]){
      can_txd0_out_ptr = &can_txd0_msg_buf[0];
    }
    while(!(SPSR & _BV(SPIF)));
    spi_port |= _BV(spi_cs0_bit);

    if(--can_txd0_cnt == 0){
      spi_port &= ~_BV(spi_cs0_bit);
      SPDR = MCP2515_BITMOD;		// TXD-IR disablen:
      while(!(SPSR & _BV(SPIF)));
      SPDR = MCP2515_CANINTE;		// Adresse
      while(!(SPSR & _BV(SPIF)));
      SPDR = MCP2515_TX0IE;		// Maske
      while(!(SPSR & _BV(SPIF)));
      SPDR = 0;				// Bit Null setzen
      can_interrupt_enable_merker &= ~MCP2515_TX0IE;
      while(!(SPSR & _BV(SPIF)));
      spi_port |= _BV(spi_cs0_bit);
    }
  }

  //*** Transmit1-Empty-IR: **********************************************

//  if (status & MCP2515_TX1IF){
//  }

 //*** ???? **************************************************************

  if (status & 0xA0){
    spi_port &= ~_BV(spi_cs0_bit);
    SPDR = MCP2515_BITMOD;		// TXD-IR disablen:
    while(!(SPSR & _BV(SPIF)));
    SPDR = MCP2515_CANINTE;		// Adresse
    while(!(SPSR & _BV(SPIF)));
    SPDR = (status & 0xAB);		// alle gesetzten IR-Flags lschen
    while(!(SPSR & _BV(SPIF)));
    SPDR = 0;				// alle IR-Flags Null setzen
    while(!(SPSR & _BV(SPIF)));
    spi_port |= _BV(spi_cs0_bit);
  }
}					// Ende IR SIG_INTERRUPT0


