
#include <avr/io.h>
#include <inttypes.h>
#include <pgmspace.h>
#include <avr/sleep.h>

#include <buskoppler.h>
#include <display.h>

#include <lcd.h>

// #include <lcd_fnt1.h>

/************************************************************************/
/*                                                                      */
/* Definition des LCD-Displays:                                         */
/*                                                                      */
/************************************************************************/
/*
typedef struct{
  unsigned char INSTR;  	// Adr. = 0000b
  unsigned char DUMMY1; 	// Adr. = 0001b
  unsigned char DUMMY2; 	// Adr. = 0010b
  unsigned char DATA;   	// Adr. = 0011b
} LCD_type, *LCD_PTR_type;
*/


/***************************************************************************/
/*                                                                         */
/* Programm LCD-Graphik:                                                   */
/* ---------------------                                                   */
/* Stellt Befehle fuer das LCD-Graphikdisplay zur Verfuegung.              */
/*                                                                         */
/***************************************************************************/

uint8_t lcd[YBSIZE][XSIZE];       /* Display-RAM-Abbild              */
uint8_t lcd_direktout;                      /* 1: Ausgabe direkt ins Display   */
                                        /* 0: Ausgabe nur ins RAM          */
#define max_kontrast 0xff
#define min_kontrast 0x80
#define max_ledpower 0x60

/*
const graphik_typ PROGMEM lcd_fnt1_0x53 = {
  0x06,
  0x08,
  {0x64,  0x92,  0x92,  0x92,  0x4C,  0x00}
};
*/

/***************************************************************************/
/*                                                                         */
/* LCD-BasisRoutinen:                                                      */
/*                                                                         */
/***************************************************************************/

const uint8_t PIXELMASK[8] PROGMEM = {0x80,0x40,0x20,0x10,0x08,0x04,0x02,0x01};
const uint8_t STARTMASK[8] PROGMEM = {0xff,0x7F,0x3F,0x1F,0x0F,0x07,0x03,0x01};
const uint8_t STOPMASK[8]  PROGMEM = {0xFF,0x80,0xC0,0xE0,0xF0,0xF8,0xFC,0xFE};

/*
void lcd_instr(uint8_t cmd){
  #ifdef RECONFIGURE_SPI
  cli();			// IRs sperren wegen umkonfiguriertem SPI
  SPCR      = 0x5D;		// CPOL = 1, CPHA = 1; 1Mhz (in double-Speed Master Mode)
  #endif
    disp_rs_port &= ~_BV(disp_rs_bit);
  spi_port &= ~_BV(disp_cs_bit);
  SPDR = cmd;
  while(!(SPSR & _BV(SPIF)));
  spi_port |= _BV(disp_cs_bit);
//  wait_us(10);
  #ifdef RECONFIGURE_SPI
  SPCR      = 0x50;		// CPOL = 0, CPHA = 0; 8Mhz (in double-Speed Master Mode)
  sei();			// IRs wieder freigeben
  #endif
}
*/



void lcd_graphik(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2){
  uint8_t y;
  if (lcd_direktout){
    y1 = y1 >> 3;
    y2 = y2 >> 3;

    for(y = y1; y <= y2; y++){
      display_set_xy(x1, YBSIZE -1 - y);
      display_xdata(&(lcd[y][x1]), x2-x1+1);
    }
  }
}


void lcd_all_graphik(void){
  uint8_t y;
  if (lcd_direktout){
    for(y = 0; y <= YBSIZE; y++){
      display_set_xy(0, YBSIZE -1 - y);
      display_xdata(&(lcd[y][0]), XSIZE - 1);
    }
  }
}


/***************************************************************************/
/*                                                                         */
/* BasisRoutinen fr LCD-Display: Reset, Init, An, Aus, Refresh, Clear     */
/*                                                                         */
/***************************************************************************/

/*
void lcd_init(void){
  lcd_x_init(&LCD1);
  lcd_x_init(&LCD2);
  lcd_direktout = 1;
}


void lcd_on(void){
  lcd_instr(&LCD1, 0xAF);
  lcd_instr(&LCD2, 0xAF);
}


void lcd_off(void){
  lcd_instr(&LCD1, 0xAE);
  lcd_instr(&LCD2, 0xAE);
}


void lcd_power_save(void){
  lcd_off();
  lcd_instr(&LCD1, 0xA5);
  lcd_instr(&LCD2, 0xA5);
}


void lcd_power_resume(void){
  lcd_instr(&LCD1, 0xA4);
  lcd_instr(&LCD2, 0xA4);
  lcd_on();
}


void lcd_reset(void){
}
*/

void lcd_refresh(void){
  uint8_t merker = lcd_direktout;

  lcd_direktout = 1;
  lcd_all_graphik();
  lcd_direktout = merker;
}


void lcd_clrscr(void){
  uint8_t x,y;
  for(y=0; y < YBSIZE; y++){
    for(x=0; x<XSIZE; x++){
      lcd[x][y] = 0;
    }
  }
  lcd_all_graphik();
}


void lcd_save(unsigned char *lcd_buf){
  unsigned char *lcd_ptr = &lcd[0][0];
  uint16_t i;

  for(i=(uint16_t)(XSIZE) * (YBSIZE); i>0; i--){
    *lcd_buf++ = *lcd_ptr++;
  }
}


void lcd_restore(unsigned char *lcd_buf){
  unsigned char *lcd_ptr = &lcd[0][0];
  uint16_t i;

  for(i=(uint16_t)(XSIZE) * (YBSIZE); i>0; i--){
    *lcd_ptr++ = *lcd_buf++;
  }
  lcd_refresh();
}


/***************************************************************************/
/*                                                                         */
/* Kontrast-Sollwert zwischen 10 und 30:                                   */
/*                                                                         */
/***************************************************************************/

/*
void lcd_kontrast(int kontrast){
  kontrast = (kontrast-10) * 12;

  if (kontrast > 255) kontrast = 255;
  if (kontrast < 0) kontrast = 0;

  GPT.PWMA = kontrast;
}
*/

/***************************************************************************/
/*                                                                         */
/* Normal-Spg. Display = 6V == 178:                                        */
/* Sollwert = 0 ... 30:                                                    */
/***************************************************************************/

void led_beleuchtung(int spg, int sollwert){  /*int duty_faktor){*/
}


/***************************************************************************/
/*                                                                         */
/* Linien setzen / lschen / invertieren:                                  */
/*                                                                         */
/***************************************************************************/

void setline(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2){
  uint8_t x, yrow, yrow1, yrow2;
  uint8_t ymask;

  #ifdef LCD_SECURE
    uint8_t temp;
    if (x1 > x2) { temp = x1; x1 = x2; x2 = temp; }
    if (y1 > y2) { temp = y1; y1 = y2; y2 = temp; }
    if (x1 > XSIZE) x1 = XSIZE;
    if (x2 > XSIZE) x2 = XSIZE;
    if (y1 > YSIZE) y1 = YSIZE;
    if (y2 > YSIZE) y2 = YSIZE;
  #endif

  yrow  = yrow1 = y1     >> 3;
  yrow2         = (y2-1) >> 3;

  do{
    if(yrow == yrow1)
      ymask =  pgm_read_byte(&STARTMASK[y1 & 0x07]);
    else
      ymask = 0xff;
    if(yrow == yrow2)
      ymask &= pgm_read_byte(&STOPMASK[y2 & 0x07]);
    for(x=x1; x<x2; x++){
      lcd[yrow][x] = lcd[yrow][x] | ymask;
    }
    yrow++;
  } while(yrow<=yrow2);

  lcd_graphik(x1, y1, x2-1, y2-1);		// auf Display ausgeben
}


void clearline(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2){
  uint8_t x, yrow, yrow1, yrow2, ymask;

  #ifdef LCD_SECURE
    uint8_t temp;
    if (x1 > x2) { temp = x1; x1 = x2; x2 = temp; }
    if (y1 > y2) { temp = y1; y1 = y2; y2 = temp; }
    if (x1 > XSIZE) x1 = XSIZE;
    if (x2 > XSIZE) x2 = XSIZE;
    if (y1 > YSIZE) y1 = YSIZE;
    if (y2 > YSIZE) y2 = YSIZE;
  #endif

  yrow  = yrow1 = y1     >> 3;
  yrow2         = (y2-1) >> 3;

  do{
    if(yrow == yrow1)
      ymask =  pgm_read_byte(&STARTMASK[y1 & 0x07]);
    else
      ymask = 0xff;
    if(yrow == yrow2)
      ymask &= pgm_read_byte(&STOPMASK[y2 & 0x07]);

    for(x=x1; x<x2; x++){
      lcd[yrow][x] = lcd[yrow][x] & ~ymask;
    }
    yrow++;
  } while(yrow<=yrow2);

  lcd_graphik(x1, y1, x2-1, y2-1);		// auf Display ausgeben
}


void invert(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2){
  uint8_t x, yrow, yrow1, yrow2, ymask;

  #ifdef LCD_SECURE
    uint8_t temp;
    if (x1 > x2) { temp = x1; x1 = x2; x2 = temp; }
    if (y1 > y2) { temp = y1; y1 = y2; y2 = temp; }
    if (x1 > XSIZE) x1 = XSIZE;
    if (x2 > XSIZE) x2 = XSIZE;
    if (y1 > YSIZE) y1 = YSIZE;
    if (y2 > YSIZE) y2 = YSIZE;
  #endif

  yrow  = yrow1 = y1     >> 3;
  yrow2         = (y2-1) >> 3;

  do{
    if(yrow == yrow1){
      ymask =  pgm_read_byte(&STARTMASK[y1 & 0x07]);
    }
    else{
      ymask = 0xff;
    }
    if(yrow == yrow2){
      ymask &= pgm_read_byte(&STOPMASK[y2 & 0x07]);
    }
    for(x=x1; x<x2; x++){
      lcd[yrow][x] = lcd[yrow][x] ^ ymask;
    }
    yrow++;
  } while(yrow<=yrow2);

  lcd_graphik(x1, y1, x2-1, y2-1);      	// auf Display ausgeben
}


/************************************************************************/
/*
/* Einzel-Pixel-Funktionen:
/*
/************************************************************************/

void setpixel(uint8_t x, uint8_t y){
  if ((x >= XSIZE) || (y >= YSIZE)){
    return;
  }

  lcd[y >> 3][x] |= pgm_read_byte(&PIXELMASK[y & 7]);
  lcd_graphik(x, y, x, y); 		// auf Display ausgeben
}


void clearpixel(uint8_t x, uint8_t y){
  if ((x >= XSIZE) || (y >= YSIZE)){
    return;
  }

  lcd[y>>3][x] &= ~pgm_read_byte(&PIXELMASK[y & 7]);
  lcd_graphik(x, y, x, y);		// auf Display ausgeben
}


/***************************************************************************/
/*                                                                         */
/* Graphikformat:                                                          */
/* --------------                                                          */
/* int x   Breite der Graphik in Pixeln                                    */
/* int y   Hhe der Graphik in Pixeln                                      */
/*                                                                         */
/* 0       1       2  . . . . . . (x-1)                                    */
/* x       x+1     x+2                                                     */
/* 2*x     2*x+1                                                           */
/* .                                                                       */
/* .                                                                       */
/* x*(y-1)                         x*y-1                                   */
/*                                                                         */
/*                                                                         */
/* Dieses Format besitzen auch die Fontdefinitionen (einzelne Charakters). */
/*                                                                         */
/***************************************************************************/

void screen_graphik(graphik_ptr g){
  uint8_t x, yrow;
  uint8_t const *gptr = &(g->g[0]);

  for(yrow = 0; yrow < YBSIZE; yrow++){
    for(x=0; x<XSIZE; x++){
      lcd[yrow][x] = pgm_read_byte(gptr++);
    }
  }
  lcd_graphik(0, 0, XSIZE-1, YSIZE-1);
}


void or_graphik(uint8_t x1, uint8_t y1, graphik_typ *g){
  uint8_t x, yrow, ygrow, yr_shift, yl_shift;
  uint8_t y_rowende;
  uint8_t const *g_byteptr = &(g->g[0]);
  uint8_t g_xsize = pgm_read_byte(&(g->x));
  uint8_t g_ysize = pgm_read_byte(&(g->y));

  if(y1 & 0x07){

    yrow = y1>>3;
    yl_shift = y1 & 0x07;
    yr_shift = 8-yl_shift;

    for(ygrow = 0; ygrow < (g_ysize+7)>>3; ygrow++){
      for(x = 0; x < g_xsize; x++){
        if((yrow   < YBSIZE) && ((x+x1) < XSIZE))
          lcd[yrow][x1+x]   |= ((pgm_read_byte(g_byteptr)) >> yl_shift);
        if((yrow+1 < YBSIZE) && ((x+x1) < XSIZE))
          lcd[yrow+1][x1+x] |= ((pgm_read_byte(g_byteptr)) << yr_shift);
        g_byteptr++;
      }
      yrow++;
    }
  }

  else{
    y_rowende = (y1 - 1 + g_ysize)>>3;

    for(yrow = y1>>3; yrow <= y_rowende; yrow++){
      for(x = 0; x < g_xsize; x++){
        if((yrow < YBSIZE) && ((x+x1) < XSIZE)){
          lcd[yrow][x1+x] |= pgm_read_byte(g_byteptr);
        }
        g_byteptr++;
      }
    }
  }

  g_xsize = x1 + g_xsize - 1; if(g_xsize >= XSIZE) g_xsize = XSIZE-1;
  g_ysize = y1 + g_ysize - 1; if(g_ysize >= YSIZE) g_ysize = YSIZE-1;
  if ((x1 < XSIZE) && (y1 < YSIZE)){
    lcd_graphik(x1, y1, g_xsize, g_ysize);	// auf Display ausgeben
  }
}


void set_graphik(uint8_t x, uint8_t y, graphik_ptr g){
  clearline(x, y, x+pgm_read_byte(&(g->x)), y+pgm_read_byte(&(g->y)));
  or_graphik(x, y, g);
}


/************************************************************************/
/*
/* Charakter- und Textfunktionen:
/*
/* Diese Funktionen benutzen set_graphik / or_graphik
/*
/************************************************************************/

uint8_t or_char(uint8_t c, uint8_t x, uint8_t y, const graphik_ptr *fptr){
  graphik_ptr g_ptr = (graphik_ptr) pgm_read_word(fptr+c);
  or_graphik(x, y, g_ptr);
  return(pgm_read_byte(&(g_ptr->x)));
}


uint8_t set_char(uint8_t c, uint8_t x, uint8_t y, const graphik_ptr *fptr){
  graphik_ptr g_ptr = (graphik_ptr) pgm_read_word(fptr+c);
  set_graphik(x, y, g_ptr);
  return(pgm_read_byte(&(g_ptr->x)));
}


uint8_t inv_char(uint8_t c, uint8_t x, uint8_t y, const graphik_ptr *fptr, uint8_t inv_oben, uint8_t inv_unten){
  graphik_ptr g_ptr = (graphik_ptr) pgm_read_word(fptr+c);
  uint8_t zeichen_hoehe = pgm_read_byte(&(g_ptr->y));
  inv_oben = y - inv_oben;
  inv_unten = y + inv_unten + zeichen_hoehe;
  if (inv_unten >= YSIZE) inv_unten = YSIZE-1;

  set_graphik(x, y, g_ptr);
  invert(x, inv_oben, x + pgm_read_byte(&(g_ptr->x)), inv_unten);    //	((graphik_typ*)(*fptr)[(unsigned char)c])->x, y2);
//  invert(10, 11, 10, 12);  //x, inv_oben, x + pgm_read_byte(&(g_ptr->x)), inv_unten);    //	((graphik_typ*)(*fptr)[(unsigned char)c])->x, y2);
//  invert(x, inv_oben, x+7, inv_unten);  //x, inv_oben, x + pgm_read_byte(&(g_ptr->x)), inv_unten);    //	((graphik_typ*)(*fptr)[(unsigned char)c])->x, y2);
  return(pgm_read_byte(&(g_ptr->x)));
}


void or_text(uint8_t *s, uint8_t x, uint8_t y, const graphik_ptr *fptr){
  uint8_t check = 0;
  while((*s) && (check < MAX_CHARSIZE)){
    x += or_char(*(s++) ,x, y, fptr);
    check++;
  }
}


void set_text(uint8_t *s, uint8_t x, uint8_t y, const graphik_ptr *fptr){
  uint8_t check = 0;
  uint8_t invert  = 0;

  while((*s) && (check < MAX_CHARSIZE)){
    if (*s == 27){
      s++;
      switch (*s++){
        case 'I': invert = *s++;    //_oben = ((*s >> 4) & 0x0f); invert_unten = ((*s++) & 0x0f);
        break;
        case 'i': invert = 0;
        break;
        case 0: return;
        break;
      }
    }
    if (invert){
      x += inv_char(*(s++), x, y, fptr, invert >> 4, invert & 0x0f);
      check++;
    }
    else{
      x += set_char(*(s++), x, y, fptr);
      check++;
    }
  }
}


/*
    x += or_char(*(s++) ,x, y, fptr);
  }
}


void set_text(uint8_t *s, uint8_t x, uint8_t y, font_typ *fptr){
  uint8_t check = MAX_CHARSIZE;
  uint8_t invert_oben  = 0;
  uint8_t invert_unten = 0;

  while((*s) && (check--)){
    if (*s == 27){
      s++;
      switch (*s++){
        case 'I': invert_oben = ((*s >> 4) & 0x0f); invert_unten = ((*s++) & 0x0f);
        break;
        case 'i': invert_unten = invert_oben = 0;
        break;
      }
    }
    if (invert_oben || invert_unten)
      x += inv_char(*s, x, y, fptr, invert_oben, invert_unten);
    else
      x += set_char(*s, x, y, fptr);
    s++;;
  }
}



int inv_char(uint8_t c, uint8_t x, uint8_t y, font_typ *fptr, uint8_t invoben, uint8_t invunten){
  uint8_t y1 = y - invoben;
  uint8_t y2 = y + invunten + (((graphik_typ*)(*fptr)[(unsigned char)c])->y);
  if (y2 >= YSIZE) y2 = YSIZE-1;

  set_graphik(x, y, (graphik_typ*)(*fptr)[(unsigned char)c]);
  invert(x, y1, x + ((graphik_typ*)(*fptr)[(unsigned char)c])->x, y2);
  return(((graphik_typ*)(*fptr)[(unsigned char)c])->x);
}


void set_text(uint8_t *s, uint8_t x, uint8_t y, font_typ *fptr){
  uint8_t check = MAX_CHARSIZE;
  uint8_t invert_oben  = 0;
  uint8_t invert_unten = 0;

  while((*s) && (check--)){
    if (*s == 27){
      s++;
      switch (*s++){
        case 'I': invert_oben = ((*s >> 4) & 0x0f); invert_unten = ((*s++) & 0x0f);
        break;
        case 'i': invert_unten = invert_oben = 0;
        break;
      }
    }
    if (invert_oben || invert_unten)
      x += inv_char(*s, x, y, fptr, invert_oben, invert_unten);
    else
      x += set_char(*s, x, y, fptr);
    s++;;
  }
}
*/

/************************************************************************/
/*
/* Breite von Zeichen und Strings ermitteln:
/*
/************************************************************************/
/*
uint8_t char_size(uint8_t c, font_typ *fptr){
  return(((graphik_typ*)(*fptr)[(unsigned char)c])->x);
}


uint8_t text_size(uint8_t *s, font_typ *fptr){
  uint8_t x = 0;
  uint8_t check = MAX_CHARSIZE;
  while((*s) && (check--)){
    if (*s == 27){
      s++;
      switch(*s++){
        case 'I': s++;
        break;
      }
    }
    x+=char_size(*(s++),fptr);
  }
  return(x);
}
*/

// end of file


