
/***************************************************************************/
/*                                                                         */
/* Programm LCD-Graphik:                                                   */
/* ---------------------                                                   */
/* Stellt Befehle fuer das LCD-Graphikdisplay zur Verfuegung.              */
/*                                                                         */
/***************************************************************************/

#define LCD_SECURE		// wenn LCD_SECURE definiert ist, dann
				// werden alle X/Y-Positionen geprft
				// und korrigiert.
				// Ohne dieses Label ist der Code etwas
				// schneller und krzer.

/***************************************************************************/
/*                                                                         */
/* Konstanten:                                                             */
/*                                                                         */
/***************************************************************************/

#define XSIZE    64	// 127	/* X-Breite des Displays                   */
#define YSIZE    64     //63	/* Y-Hhe in Pixeln                        */
#define YBSIZE   8              /* Y-Hhe in Byte                          */

#define MAX_CHARSIZE 60         /* maximale Char-Anzahl in Strings         */


/***************************************************************************/
/*                                                                         */
/* Typdefinitionen:                                                        */
/*                                                                         */
/***************************************************************************/

typedef const struct{     /* Definition einer Graphik                */
  uint8_t x;
  uint8_t y;
  uint8_t g[];		// XSIZE*YBSIZE];
} graphik_typ, *graphik_ptr;

// typedef const graphik_ptr font_typ[];	// void *font_typ[];       /* Font-Definition                         */


/***************************************************************************/
/*                                                                         */
/* Globale Variablen:                                                      */
/*                                                                         */
/***************************************************************************/

extern uint8_t lcd[YBSIZE][XSIZE];	/* Display-RAM-Abbild              */
extern uint8_t lcd_direktout;               /* 1: Ausgabe direkt ins Display   */
                                        /* 0: Ausgabe nur ins RAM          */


extern const graphik_typ lcd_fnt1_0x53;



/***************************************************************************/
/*                                                                         */
/* BasisRoutinen fr LCD-Display: Reset, Init, An, Aus, Refresh, Clear     */
/*                                                                         */
/***************************************************************************/
/*
void lcd_init(void);

void lcd_on(void);

void lcd_off(void);

void lcd_power_save(void);

void lcd_power_resume(void);

void lcd_refresh(void);

void lcd_clrscr(void);

void lcd_save(unsigned char *lcd_buf);

void lcd_restore(unsigned char *lcd_buf);

void lcd_kontrast(int kontrast);

void led_beleuchtung(int spg, int sollwert); /* (int duty_faktor);
*/

void lcd_graphik(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2);


/***************************************************************************/
/*                                                                         */
/* Zeichenfunktionen:                                                      */
/*                                                                         */
/***************************************************************************/


void setline(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2);

void clearline(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2);

void invert(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2);


/************************************************************************/
/*
/* Einzel-Pixel-Funktionen:
/*
/************************************************************************/

void setpixel(uint8_t x, uint8_t y);

void clearpixel(uint8_t x, uint8_t y);


/***************************************************************************/
/*                                                                         */
/* Graphik-Befehle:                                                        */
/* ----------------                                                        */
/* int x   Breite der Graphik in Pixeln                                    */
/* int y   Hhe der Graphik in Pixeln                                      */
/*                                                                         */
/* 0       1       2  . . . . . . (x-1)                                    */
/* x       x+1     x+2                                                     */
/* 2*x     2*x+1                                                           */
/* .                                                                       */
/* .                                                                       */
/* x*(y-1)                         x*y-1                                   */
/*                                                                         */
/*                                                                         */
/* Dieses Format besitzen auch die Fontdefinitionen (einzelne Charakters). */
/*                                                                         */
/***************************************************************************/

void screen_graphik(graphik_ptr g);

void or_graphik(uint8_t x1, uint8_t y1, const graphik_typ *g);

/*
void screen_graphik(graphik_ptr g);

void or_graphik(int x1, int y1, graphik_ptr g);

void set_graphik(int x, int y, graphik_ptr g);
*/

/***************************************************************************/
/*                                                                         */
/* String- und Charakter-Funktionen:                                       */
/*                                                                         */
/***************************************************************************/

uint8_t or_char(uint8_t c, uint8_t x, uint8_t y, const graphik_ptr *fptr);

uint8_t set_char(uint8_t c, uint8_t x, uint8_t y, const graphik_ptr *fptr);

uint8_t inv_char(uint8_t c, uint8_t x, uint8_t y, const graphik_ptr *fptr, uint8_t inv_oben, uint8_t inv_unten);

void or_text(uint8_t *s, uint8_t x, uint8_t y, const graphik_ptr *fptr);

void set_text(uint8_t *s, uint8_t x, uint8_t y, const graphik_ptr *fptr);


/*
int or_char(char c, int x,int y,font_typ *fptr);

void or_text(char *s,int x,int y,font_typ *fptr);

int set_char(char c, int x, int y, font_typ *fptr);

void set_text(char *s, int x, int y, font_typ *fptr);

int char_size(char c, font_typ *fptr);

int text_size(char *s, font_typ *fptr);
*/

