
#include <avr/io.h>
#include <inttypes.h>
#include <pgmspace.h>
#include <avr/sleep.h>

#include <spi.h>
#include <can.h>
#include <mcp2515.h>
#include <buskoppler.h>

#include <avr/interrupt.h>
#include <avr/signal.h>


#include <scheduler.h>

#include <taster.h>
#include <relais.h>
#include <dimmer.h>

#include <tab.h>

/************************************************************************/
/*
/* Scheduler.c
/*
/************************************************************************/

#define _DEBUG 1

#define QUARZFREQUENZ	4000000


/************************************************************************/
/*
/* Listen der verwendeten ffentlichen Funktionen:
/*
/************************************************************************/

void init_nothing(const module_parm_typ *my_module){
}

void timer_nothing(const module_parm_typ *my_module){
}

void msg_nothing(const module_parm_typ *my_module, can_msg_typ *my_msg){
}


#define typ_anz	7

PROGMEM init_function_typ init_function_table[typ_anz+1] =
  {init_nothing, init_nothing, init_taster, init_taster, init_taster, init_relais, init_jalousie, init_dimmer};

PROGMEM timer_function_typ timer_function_table[typ_anz+1] =
  {timer_nothing, timer_nothing, timer_taster, timer_taster, timer_taster, timer_relais, timer_jalousie, timer_dimmer};

PROGMEM msg_function_typ msg_function_table[typ_anz+1] =
  {msg_nothing, msg_nothing, msg_taster, msg_taster, msg_taster, msg_relais, msg_jalousie, msg_dimmer};


/************************************************************************/
/*
/* Parameter und Variablen der Module:
/*
/************************************************************************/

volatile uint8_t msg_cnt;
volatile uint8_t timer_cnt;

// module_var_typ module_var[module_anz];


/************************************************************************/
/*
/*
/*
/************************************************************************/

void send_byte_msg(const module_parm_typ *my_module, uint8_t action){
  uint8_t i;
  can_adress_typ my_adr;
  can_msg_typ *rxd_msg_ptr;
  can_msg_typ *txd_msg_ptr;

  for(i=0;i<12;i++){
    memcpy_P(&my_adr, &(my_module->adr[i][0]), 4);

    //*** Sende-Adresse == 0 ist Abbruchbedingung (alle msgs versandt):
    if (my_adr[can_sende_adress] == 0){
      break;
    }

    if (my_adr[can_rec_adress] == buskoppler_adress){
      // msg ist an diesen Buskoppler adressiert: in Eingangs-Puffer kopieren
      if (can_assign_rxd0_msg(&rxd_msg_ptr)){
        (*rxd_msg_ptr)[can_rec_adress]    = my_adr[can_rec_adress];
        (*rxd_msg_ptr)[can_function_bits] = my_adr[can_function_bits];
        (*rxd_msg_ptr)[can_module_adress] = my_adr[can_module_adress];
        (*rxd_msg_ptr)[can_sende_adress]  = buskoppler_adress;
        (*rxd_msg_ptr)[can_msg_size]      = 1;
        (*rxd_msg_ptr)[can_msg_data] 	  = action;
      }
    }
    else{
      // msg geht an externen Buskoppler: in Ausgangs-Puffer kopieren
      txd_msg_ptr                 	= can_assign_txd0_msg();
      (*txd_msg_ptr)[can_rec_adress]    = my_adr[can_rec_adress];
      (*txd_msg_ptr)[can_function_bits] = my_adr[can_function_bits] | can_extended_id;
      (*txd_msg_ptr)[can_module_adress] = my_adr[can_module_adress];
      (*txd_msg_ptr)[can_sende_adress]  = buskoppler_adress;
      (*txd_msg_ptr)[can_msg_size]      = 1;
      (*txd_msg_ptr)[can_msg_data]	= action;
      can_put_txd0_msg();
    }
  }
}


/*
void send_str_msg(const module_parm_typ *my_module, uint8_t *str_ptr){
}
*/


/************************************************************************/
/*
/* Timer-Interrupt (als Zeitbasis)
/* Vorsicht, bei nderung der Taktfrequenz Timer entsprechend nachfhren!
/*
/************************************************************************/

void tmr_init(void){
  TCCR0 =  (_BV(WGM01) | _BV(CS02) | _BV(CS00)); // CTC-Mode, Prescaler = 1024
  TIMSK |= _BV(OCIE0);
  OCR0  =  QUARZFREQUENZ / 1024 / 50;	//156 bei 8Mhz, 78 bei 4Mhz (20ms Timer-Rate)
}


SIGNAL(SIG_OUTPUT_COMPARE0){
  timer_cnt++;
}


/************************************************************************/
/*
/*
/*
/************************************************************************/

void scheduler(void){
  can_msg_typ *msg_ptr;
  uint8_t i;
  volatile uint8_t module_nr;
  volatile uint8_t module_typ;
  msg_function_typ msg_function_ptr;
  timer_function_typ timer_function_ptr;
  init_function_typ init_function_ptr;
  uint8_t led_cnt = 0;

  // init_scheduler();

  for(i=0; i<module_anz;i++){
    module_typ = pgm_read_byte(&module_parm[i].typ);
    if ((module_typ > typ_not_assigned) && (module_typ <= typ_anz)){
      init_function_ptr = (init_function_typ)pgm_read_word(&init_function_table[module_typ]);
      init_function_ptr(&module_parm[i]);
    }
  }

  tmr_init();
  sei();

  while(1){
    if (can_rxd0_cnt > 0){
      led_port  |= _BV(led_bit);
      led_cnt    =  25;		// bei msg-Empfang 500ms LED einschalten

      msg_ptr    = can_get_rxd0_msg();
      // nur msgs fr diesen Buskoppler bearbeiten:
      if (*msg_ptr[can_rec_adress] == buskoppler_adress){
        module_nr  = (*msg_ptr)[can_module_adress] & can_module_mask;
        module_typ = pgm_read_byte(&module_parm[module_nr].typ);
        if ((module_typ > typ_not_assigned) && (module_typ <= typ_anz)){
          msg_function_ptr = (msg_function_typ)pgm_read_word(&msg_function_table[module_typ]);
          msg_function_ptr(&module_parm[module_nr], msg_ptr);
        }
      }
      can_free_rxd0_msg();
    }

    if (timer_cnt > 0){
      timer_cnt--;

      led_cnt--;
      if (led_cnt == 2){
        led_port |= _BV(led_bit);
      }
      if (led_cnt == 0){
        led_port &= ~_BV(led_bit);
        led_cnt = 200;			// blinken alle 4s
      }

      for(i=0; i<module_anz;i++){
        if(module_var[i].timer){
          if((--module_var[i].timer) == 0){
            module_typ         = pgm_read_byte(&module_parm[i].typ);
            if ((module_typ > typ_not_assigned) && (module_typ <= typ_anz)){
              timer_function_ptr = (timer_function_typ)pgm_read_word(&timer_function_table[module_typ]);
              timer_function_ptr(&module_parm[i]);
            }
          }
        }
      }
      spi_relais(rel1, rel2, rel3); // alle 20ms Relais "auffrischen"
    }


    #if _DEBUG
      //************************************************** bei Debuggen kein Idle-Mode, sonst hat AVRStudio ein Problem
    #else
      //************************************************** in den Idle-Mode gehen und auf nchsten IR warten:
      cli();						// IR sperren
      if ((can_rxd0_cnt == 0) && (timer_cnt == 0)){	// testen, ob rxd_cnt oder timer_cnt gendert wurden
        MCUCR &= ~(_BV(SE) | _BV(SM2) | _BV(SM1) | _BV(SM0));
        MCUCR |= _BV(SE);
        sei();
        asm("sleep");
        MCUCR &= ~_BV(SE);
      }
      sei();
    #endif
  }
}


// End of file

